/***
 *  Copyright (C) 2009 Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "instrumentconfig.h"
#include "ui_instrumentconfig.h"


InstrumentConfig::InstrumentConfig (QWidget *parent) :
    QFrame (parent),
    m_ui(new Ui::InstrumentConfig) {
    m_ui->setupUi (this);

    initBar (m_ui->bar_Scale,       0.10f, 2.0f,  1.0f, tr ("Instrument size."));
    initBar (m_ui->bar_Sensitivity, 0.01f, 1.0f,  0.8f, tr ("Sensitivity to paper texture."));
    initBar (m_ui->bar_Jitter,      0.00f, 32.0f, 0.0f, tr ("Instrument jitter."));

    m_Current = NULL;

    QObject::connect (&m_UpdateTimer, SIGNAL(timeout()), this, SLOT(barsApply()));

    // Me be floating.
    setWindowFlags (Qt::Popup | Qt::Tool);
}

InstrumentConfig::~InstrumentConfig (void) {
    SAFE_RELEASE (m_Current);
    delete m_ui;
}

void InstrumentConfig::checkSetValue (ClickBar * rot, float value) {
    if (rot) {
        rot->setEnabled (TRUE);
        rot->setPosition (value);
    }
}

void InstrumentConfig::initBar (ClickBar * rot, float minv, float maxv, float curv,
                       QString tooltip) {
    if (rot) {
        rot->setBounds(minv, maxv);
        rot->setPosition(curv);
        rot->setToolTip (tooltip);

        QObject::connect (rot, SIGNAL(positionChanged(float)), this, SLOT(barsChanged(float)));
    }
}

void InstrumentConfig::changeEvent (QEvent *e) {
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void InstrumentConfig::rereadControls (void) {
    if (m_Current) {
        checkSetValue (m_ui->bar_Scale,       MATH::CLAMP (m_Current->scaleModifier (),              0.25f, 2.00f));
        checkSetValue (m_ui->bar_Sensitivity, MATH::CLAMP (m_Current->textureSensitivityModifier (), 0.00f, 1.00f));
        checkSetValue (m_ui->bar_Jitter,      MATH::CLAMP (m_Current->jitter (),                     0.00f, 16.0f));
    }
}

void InstrumentConfig::rewriteControls (void) {
    if (m_Current) {
        if (m_ui->bar_Scale)       m_Current->setScaleModifier (m_ui->bar_Scale->position ());
        if (m_ui->bar_Sensitivity) m_Current->setTextureSensitivityModifier (MATH::CLAMP (m_ui->bar_Sensitivity->position (), 0.0f, 1.0f));
        if (m_ui->bar_Jitter)      m_Current->applyJitter (MATH::CLAMP (m_ui->bar_Jitter->position (), 0.0f, 16.0f));
    }
}

void InstrumentConfig::barsChanged (float) {
    m_UpdateTimer.stop ();
    m_UpdateTimer.start (128);
}

void InstrumentConfig::barsApply (void) {
    m_UpdateTimer.stop ();
    rewriteControls ();

    if (m_Current) {
        emit instrumentSizeChanged (m_Current,
                                    m_Current->currentPrimitiveWidth () / 2);
    }
}

void InstrumentConfig::popup (QPoint p) {
    move (p);
    show ();
}

void InstrumentConfig::setPressureCurve (ISpline * curve) {
    if (m_ui->curvePressure) {
        m_ui->curvePressure->setSource (curve);
    }
}
