/***
 *  Copyright (C) 2009 Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef INSTRUMENTCONFIG_H
#define INSTRUMENTCONFIG_H

#include "commdefs.h"
#include "commtypes.h"
#include "commmath.h"
#include "refcounter.h"

#   include "iInstrument.h"
#   include "iInstrumentProxy.h"
#   include "iInstrumentList.h"
#   include "iCanvasLayer.h"
#   include "clickbar.h"

#   include <QtGui/QFrame>
#   include <QTimer>

namespace Ui {
    class InstrumentConfig;
}

class InstrumentConfig : public QFrame {
    Q_OBJECT

public:
    InstrumentConfig  (QWidget *parent = 0);
    ~InstrumentConfig (void);

signals:
    /**
     * Emitted when user changes
     * the instrument size.
     */
    void instrumentSizeChanged (IInstrumentProxy * instrument,
                                int radius);
    void closed (void);

public slots:
    void popup            (QPoint p);
    void rereadControls   (void);
    void rewriteControls  (void);
    void setPressureCurve (ISpline * curve);

    void setInstrument   (IInstrumentProxy * instrument) {
        SAFE_RELEASE (m_Current);

        m_Current = instrument;
        m_Current->addRef ();
    }

private slots:
    void barsChanged (float);
    void barsApply   (void);

protected:
    void changeEvent (QEvent *e);
    virtual void closeEvent (QCloseEvent * e) {
        Q_UNUSED (e);
        emit closed ();
    }

private:
    Ui::InstrumentConfig * m_ui;

    QTimer m_UpdateTimer;

    /**
     * Current instrument, if set.
     */
    IInstrumentProxy * m_Current;

    void checkSetValue (ClickBar * rot, float value);
    void initBar    (ClickBar * rot, float minv, float maxv, float curv,
                     QString tooltip = "");
};

#endif // INSTRUMENTCONFIG_H
