/* Qaquarelle
 * Copyright (C) 2009 Vasiliy Makarov <drmoriarty.0@gmail.com>
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "keyedit.h"

KeyEdit::KeyEdit(QWidget *parent) : QLineEdit(parent)
{
	_key = -1;
}

void KeyEdit::keyPressEvent ( QKeyEvent * event )
{
	if(_key == Qt::Key_Backspace || _key == Qt::Key_Delete) {
		if(_key > 0) {
			_key = 0;
			setText("");
		} else if(_key == 0) {
			_key = -1;
			setText("none");
		}
	} else {
		switch(event->key()) {
		case Qt::Key_Shift:
		case Qt::Key_Control:
		case Qt::Key_Alt:
		case Qt::Key_Meta:
			_key = event->modifiers();
			break;
		default:
			_key = event->key() | event->modifiers();
		}
		setText( keyToString(_key));
	}
}

void KeyEdit::setKey(int k)
{
	_key = k;
	if(_key < 0)
		setText("none");
	else if(!_key)
		setText("");
	else
		setText(keyToString(_key));
}

int KeyEdit::key()
{
	return _key;
}

QString KeyEdit::keyToString(int key) {
	QString str;
#define append(s1, s2) {s1.size()?s1=s1+" + "+s2:s1=s2;}
	if(key & Qt::ShiftModifier) {
		append(str, "Shift");
		key &= ~Qt::ShiftModifier;
	}
	if(key & Qt::ControlModifier) {
		append(str, "Control");
		key &= ~Qt::ControlModifier;
	}
	if(key & Qt::AltModifier) {
		append(str, "Alt");
		key &= ~Qt::AltModifier;
	}
	if(key & Qt::MetaModifier) {
		append(str, "Meta");
		key &= ~Qt::MetaModifier;
	}

	switch (key) {
	case 0: break;
	case Qt::Key_Shift: //append(str, "Shift"); break;
	case Qt::Key_Control: //append(str, "Control"); break;
	case Qt::Key_Alt: //append(str, "Alt"); break;
	case Qt::Key_Meta: //append(str, "Meta"); break;
		break;
	case Qt::Key_Escape: append(str, "Esc"); break;
	case Qt::Key_Tab: append(str, "Tab"); break;
	case Qt::Key_Backtab: append(str, "Backtab"); break;
	case Qt::Key_Backspace: append(str, "Backspace"); break;
	case Qt::Key_Return: append(str, "Return"); break;
	case Qt::Key_Enter: append(str, "Enter"); break;
	case Qt::Key_Insert: append(str, "Insert"); break;
	case Qt::Key_Delete: append(str, "Delete"); break;
	case Qt::Key_Pause: append(str, "Pause"); break;
	case Qt::Key_Print: append(str, "Print"); break;
	case Qt::Key_SysReq: append(str, "SysReq"); break;
	case Qt::Key_Clear: append(str, "Clear"); break;
	case Qt::Key_Home: append(str, "Home"); break;
	case Qt::Key_End: append(str, "End"); break;
	case Qt::Key_Left: append(str, "Left"); break;
	case Qt::Key_Up: append(str, "Up"); break;
	case Qt::Key_Right: append(str, "Right"); break;
	case Qt::Key_Down: append(str, "Down"); break;
	case Qt::Key_PageUp: append(str, "PageUp"); break;
	case Qt::Key_PageDown: append(str, "PageDown"); break;
	case Qt::Key_CapsLock: append(str, "CapsLock"); break;
	case Qt::Key_NumLock: append(str, "NumLock"); break;
	case Qt::Key_ScrollLock: append(str, "ScrollLock"); break;
	case Qt::Key_F1: append(str, "F1"); break;
	case Qt::Key_F2: append(str, "F2"); break;
	case Qt::Key_F3: append(str, "F3"); break;
	case Qt::Key_F4: append(str, "F4"); break;
    case Qt::Key_F5: append(str, "F5"); break;
    case Qt::Key_F6: append(str, "F6"); break;
    case Qt::Key_F7: append(str, "F7"); break;
    case Qt::Key_F8: append(str, "F8"); break;
    case Qt::Key_F9: append(str, "F9"); break;
    case Qt::Key_F10: append(str, "F10"); break;
    case Qt::Key_F11: append(str, "F11"); break;
    case Qt::Key_F12: append(str, "F12"); break;
    case Qt::Key_F13: append(str, "F13"); break;
    case Qt::Key_F14: append(str, "F14"); break;
    case Qt::Key_F15: append(str, "F15"); break;
    case Qt::Key_F16: append(str, "F16"); break;
    case Qt::Key_F17: append(str, "F17"); break;
    case Qt::Key_F18: append(str, "F18"); break;
    case Qt::Key_F19: append(str, "F19"); break;
    case Qt::Key_F20: append(str, "F20"); break;
    case Qt::Key_F21: append(str, "F21"); break;
    case Qt::Key_F22: append(str, "F22"); break;
    case Qt::Key_F23: append(str, "F23"); break;
    case Qt::Key_F24: append(str, "F24"); break;
    case Qt::Key_F25: append(str, "F25"); break;
    case Qt::Key_F26: append(str, "F26"); break;
    case Qt::Key_F27: append(str, "F26"); break;
    case Qt::Key_F28: append(str, "F28"); break;
    case Qt::Key_F29: append(str, "F29"); break;
    case Qt::Key_F30: append(str, "F30"); break;
    case Qt::Key_F31: append(str, "F31"); break;
    case Qt::Key_F32: append(str, "F32"); break;
    case Qt::Key_F33: append(str, "F33"); break;
    case Qt::Key_F34: append(str, "F34"); break;
    case Qt::Key_F35: append(str, "F35"); break;
    case Qt::Key_Super_L: append(str, "LeftSuper"); break;
    case Qt::Key_Super_R: append(str, "RightSuper"); break;
    case Qt::Key_Menu: append(str, "Menu"); break;
    case Qt::Key_Hyper_L: append(str, "LeftHyper"); break;
    case Qt::Key_Hyper_R: append(str, "RightHyper"); break;
    case Qt::Key_Help: append(str, "Help"); break;
    case Qt::Key_Direction_L: append(str, "LeftDirection"); break;
    case Qt::Key_Direction_R: append(str, "RightDirection"); break;
    case Qt::Key_Space: append(str, "Space"); break;
    case Qt::Key_Exclam: append(str, "Exclam"); break;
    case Qt::Key_QuoteDbl: append(str, "QuoteDbl"); break;
    case Qt::Key_NumberSign: append(str, "NumberSign"); break;
    case Qt::Key_Dollar: append(str, "Dollar"); break;
    case Qt::Key_Percent: append(str, "Percent"); break;
    case Qt::Key_Ampersand: append(str, "Ampersand"); break;
    case Qt::Key_Apostrophe: append(str, "Apostrophe"); break;
    case Qt::Key_ParenLeft: append(str, "LeftParen"); break;
    case Qt::Key_ParenRight: append(str, "RightParen"); break;
    case Qt::Key_Asterisk: append(str, "Asterisk"); break;
    case Qt::Key_Plus: append(str, "Plus"); break;
    case Qt::Key_Comma: append(str, "Comma"); break;
    case Qt::Key_Minus: append(str, "Minus"); break;
    case Qt::Key_Period: append(str, "Period"); break;
    case Qt::Key_Slash: append(str, "Slash"); break;
    case Qt::Key_0: append(str, "0"); break;
    case Qt::Key_1: append(str, "1"); break;
    case Qt::Key_2: append(str, "2"); break;
    case Qt::Key_3: append(str, "3"); break;
    case Qt::Key_4: append(str, "4"); break;
    case Qt::Key_5: append(str, "5"); break;
    case Qt::Key_6: append(str, "6"); break;
    case Qt::Key_7: append(str, "7"); break;
    case Qt::Key_8: append(str, "8"); break;
    case Qt::Key_9: append(str, "9"); break;
    case Qt::Key_Colon: append(str, "Colon"); break;
    case Qt::Key_Semicolon: append(str, "Semicolon"); break;
    case Qt::Key_Less: append(str, "Less"); break;
    case Qt::Key_Equal: append(str, "Equal"); break;
    case Qt::Key_Greater: append(str, "Greater"); break;
    case Qt::Key_Question: append(str, "Question"); break;
    case Qt::Key_At: append(str, "At"); break;
    case Qt::Key_A: append(str, "A"); break;
    case Qt::Key_B: append(str, "B"); break;
    case Qt::Key_C: append(str, "C"); break;
    case Qt::Key_D: append(str, "D"); break;
    case Qt::Key_E: append(str, "E"); break;
    case Qt::Key_F: append(str, "F"); break;
    case Qt::Key_G: append(str, "G"); break;
    case Qt::Key_H: append(str, "H"); break;
    case Qt::Key_I: append(str, "I"); break;
    case Qt::Key_J: append(str, "J"); break;
    case Qt::Key_K: append(str, "K"); break;
    case Qt::Key_L: append(str, "L"); break;
    case Qt::Key_M: append(str, "M"); break;
    case Qt::Key_N: append(str, "N"); break;
    case Qt::Key_O: append(str, "O"); break;
    case Qt::Key_P: append(str, "P"); break;
    case Qt::Key_Q: append(str, "Q"); break;
    case Qt::Key_R: append(str, "R"); break;
    case Qt::Key_S: append(str, "S"); break;
    case Qt::Key_T: append(str, "T"); break;
    case Qt::Key_U: append(str, "U"); break;
    case Qt::Key_V: append(str, "V"); break;
    case Qt::Key_W: append(str, "W"); break;
    case Qt::Key_X: append(str, "X"); break;
    case Qt::Key_Y: append(str, "Y"); break;
    case Qt::Key_Z: append(str, "Z"); break;
    case Qt::Key_BracketLeft: append(str, "LeftBracket"); break;
    case Qt::Key_Backslash: append(str, "Backslash"); break;
    case Qt::Key_BracketRight: append(str, "RightBracket"); break;
    case Qt::Key_AsciiCircum: append(str, "Circum"); break;
    case Qt::Key_Underscore: append(str, "Underscore"); break;
    case Qt::Key_QuoteLeft: append(str, "LeftQuote"); break;
    case Qt::Key_BraceLeft: append(str, "LeftBrace"); break;
    case Qt::Key_Bar: append(str, "Bar"); break;
    case Qt::Key_BraceRight: append(str, "RightBrace"); break;
    case Qt::Key_AsciiTilde: append(str, "Tilde"); break;

    case Qt::Key_nobreakspace: append(str, "NobreakSpace"); break;
    case Qt::Key_exclamdown: append(str, "ExclamDown"); break;
    case Qt::Key_cent: append(str, "Cent"); break;
    case Qt::Key_sterling: append(str, "Sterling"); break;
    case Qt::Key_currency: append(str, "Currency"); break;
    case Qt::Key_yen: append(str, "Yen"); break;
    case Qt::Key_brokenbar: append(str, "BrokenBar"); break;
    case Qt::Key_section: append(str, "Section"); break;
    case Qt::Key_diaeresis: append(str, "Diaeresis"); break;
    case Qt::Key_copyright: append(str, "Copyright"); break;
    case Qt::Key_ordfeminine: append(str, "OrdFeminine"); break;
    case Qt::Key_guillemotleft: append(str, "LeftGuillemot"); break;
    case Qt::Key_notsign: append(str, "NotSign"); break;
    case Qt::Key_hyphen: append(str, "Hyphen"); break;
    case Qt::Key_registered: append(str, "Registered"); break;
    case Qt::Key_macron: append(str, "Macon"); break;
    case Qt::Key_degree: append(str, "Degree"); break;
    case Qt::Key_plusminus: append(str, "Plusminus"); break;
    case Qt::Key_twosuperior: append(str, "TwoSuperior"); break;
    case Qt::Key_threesuperior: append(str, "TreeSuperior"); break;
    case Qt::Key_acute: append(str, "Acute"); break;
    case Qt::Key_mu: append(str, "Mu"); break;
    case Qt::Key_paragraph: append(str, "Paragraph"); break;
    case Qt::Key_periodcentered: append(str, "CenteredPeriod"); break;
    case Qt::Key_cedilla: append(str, "Cedilla"); break;
    case Qt::Key_onesuperior: append(str, "OneSuperior"); break;
    case Qt::Key_masculine: append(str, "Masculine"); break;
    case Qt::Key_guillemotright: append(str, "RightQuillemot"); break;
    case Qt::Key_onequarter: append(str, "OneQuarter"); break;
    case Qt::Key_onehalf: append(str, "OneHalf"); break;
    case Qt::Key_threequarters: append(str, "ThreeQuarters"); break;
    case Qt::Key_questiondown: append(str, "DownQuestion"); break;
    case Qt::Key_Agrave: append(str, "AGrave"); break;
    case Qt::Key_Aacute: append(str, "AAcute"); break;
    case Qt::Key_Acircumflex: append(str, "ACircumflex"); break;
    case Qt::Key_Atilde: append(str, "ATilde"); break;
    case Qt::Key_Adiaeresis: append(str, "ADiaeresis"); break;
    case Qt::Key_Aring: append(str, "ARing"); break;
    case Qt::Key_AE: append(str, "AE"); break;
    case Qt::Key_Ccedilla: append(str, "CCedilla"); break;
    case Qt::Key_Egrave: append(str, "EGrave"); break;
    case Qt::Key_Eacute: append(str, "EAcute"); break;
    case Qt::Key_Ecircumflex: append(str, "ECircumflex"); break;
    case Qt::Key_Ediaeresis: append(str, "EDiaeresis"); break;
    case Qt::Key_Igrave: append(str, "IGrave"); break;
    case Qt::Key_Iacute: append(str, "IAcute"); break;
    case Qt::Key_Icircumflex: append(str, "ICircumflex"); break;
    case Qt::Key_Idiaeresis: append(str, "IDiaeresis"); break;
    case Qt::Key_ETH: append(str, "ETH"); break;
    case Qt::Key_Ntilde: append(str, "NTilde"); break;
    case Qt::Key_Ograve: append(str, "Ograve"); break;
    case Qt::Key_Oacute: append(str, "OAcute"); break;
    case Qt::Key_Ocircumflex: append(str, "OCircumflex"); break;
    case Qt::Key_Otilde: append(str, "OTilde"); break;
    case Qt::Key_Odiaeresis: append(str, "Odiaeresis"); break;
    case Qt::Key_multiply: append(str, "Multiply"); break;
    case Qt::Key_Ooblique: append(str, "Ooblique"); break;
    case Qt::Key_Ugrave: append(str, "UGrave"); break;
    case Qt::Key_Uacute: append(str, "UAcute"); break;
    case Qt::Key_Ucircumflex: append(str, "UCircumflex"); break;
    case Qt::Key_Udiaeresis: append(str, "Udiaeresis"); break;
    case Qt::Key_Yacute: append(str, "YAcute"); break;
    case Qt::Key_THORN: append(str, "THORN"); break;
    case Qt::Key_ssharp: append(str, "SSharp"); break;
    case Qt::Key_division: append(str, "Division"); break;
    case Qt::Key_ydiaeresis: append(str, "YDiaeresis"); break;

	default:
		append(str, QString("#%1").arg(QString::number(key & 0xFF, 16)));
	}
#undef append
	return str;
}

