/***************************************************************************
 *  Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once
#include "commdefs.h"
#include "commtypes.h"
#include "refcounter.h"
#include "iArrayGrid.h"


class Patch : public IArrayGridPatch {
    sRect    m_Rect;
    IArray * m_Data;
    int      m_X;
    int      m_Y;
    int      m_Status;

public:
    Patch (void) {
        m_Data        = NULL;
        m_Rect.left   = 0;
        m_Rect.right  = 0;
        m_Rect.top    = 0;
        m_Rect.bottom = 0;
        m_Status      = PATCHSTAT_NONE;
    }

    /**
     * X-offset of the patch in grid.
     */
    virtual int cellX (void) const {
        return m_X;
    }

    /**
     * Y-offset of the patch in grid.
     */
    virtual int cellY (void) const {
        return m_Y;
    }

    /**
     * Rectangle in pixels occupied by the patch.
     */
    virtual const sRect * rect (void) {
        return &m_Rect;
    }

    /**
     * Get the patch as IArray interface.
     */
    virtual IArray * data (void) {
        return m_Data;
    }

    /**
     * Get the current status of the patch.
     */
    virtual int status (void) const {
        return m_Status;
    }

    /**
     * Update the status infromation
     * according to current data.
     */
    virtual void checkStatus (void);

    /**
     * Mark the patch as touched by
     * instrument. This resets its status.
     */
    virtual void touched (void) {
        m_Status = PATCHSTAT_UNKNOWN;
    }

    void setCellX (int x) {
        m_X = x;
    }
    void setCellY (int y) {
        m_Y = y;
    }
    void setData (IArray * data) {
        m_Data = data;
    }
    void setRect (const sRect * rect) {
        m_Rect.left   = rect->left;
        m_Rect.top    = rect->top;
        m_Rect.right  = rect->right;
        m_Rect.bottom = rect->bottom;
    }
    void setLeft (int left) {
        m_Rect.left = left;
    }
    void setRight (int right) {
        m_Rect.right = right;
    }
    void setTop (int top) {
        m_Rect.top = top;
    }
    void setBottom (int bottom) {
        m_Rect.bottom = bottom;
    }
};
